function ampe
% The software AMP Eigensolver is an eigensolver for self-adjoint and elliptic partial differential operators in 2D domains.
% Copyright 2014  Ming Zhou & Klaus Neymeyr
% Universitaet Rostock, Institut fuer Mathematik
%     
% The software can be used for academic, research and other similar noncommercial uses.
% The user acknowledges that this software is still in the development stage and
% that it is provided by the copyright holders and contributors "as is" and any express
% or implied warranties, fitness for a particular purpose are disclaimed. 
% In no event shall the copyright owner or contributors be liable for any direct,
% indirect, incidental, special, exemplary, or consequential damages.
%     
% The copyright holders provide no reassurances that the source code provided does not
% infringe any patent, copyright, or any other intellectual property rights of third
% parties. The copyright holders disclaim any liability to any recipient for claims
% brought against recipient by any third party for infringement of that parties
% intellectual property rights.
%     
% The user has to respect the lapack license, see
% http://www.netlib.org/lapack/LICENSE.txt

sc=get(0,'ScreenSize'); s1=sc(3); s2=s1/2;
s=[s1*0.05 (sc(4)-s2)/2 s1*0.9 s2];
fig = figure('Position',s,'MenuBar','none','NumberTitle','off',...
    'Name',['Adaptive Multigrid Preconditioned Eigensolver (AMPEigensolver) ',...
    char(169),' 2014  Ming Zhou  &  Klaus Neymeyr ']);
bgc=[1 1 1]; bgc1=[0.7 0.7 0.7]; cl1=[0 0.3 1];
fst=1; uct=1;
if ismac, fst=0.75; uct=1.25; end
if ispc, fst=0.85; uct=1.2; end
domainselected=0; efdrawn=0;
x=[]; y=[];
v=[]; v1=[];
e=[]; ep=[];
tr=[]; trp=[];
cc=[]; ci=[];
inx=[]; iny=[];
invt=[]; inv1=[];
ine=[]; inep=[];
intr=[]; intrp=[];
inh=[]; inaxis=[];
nn=[]; tt=[]; tt1=[];
n=[]; em=[]; m=[];
on=[]; u=[]; thetav=[];
efi=[]; rei=[];
lv=[]; rn=[];
sp=[]; spv=[]; spf=[];
fs=[]; fs1=[]; fs2=[];
tempf=[]; evj=[]; tempfv=[];
n1=[]; n2=[];
lambda=[]; axisv=[];

bgp = uipanel('BackgroundColor',bgc,...
    'Units','normalized','Position',[0 0 1 1]); %,...
    %'ButtonDownFcn',{@bgp_ButtonDownFcn},'ResizeFcn',{@bgp_ResizeFcn}

icp = uipanel('Parent',bgp,'Title','Initialization & Computation',...
    'BackgroundColor',bgc,...
    'Units','normalized','Position',[.01 .02 .56 .96]);

evp = uicontrol('Parent',icp,'BackgroundColor',bgc,'Style','text',...
    'String',' Eigenvalue problem for the Laplace Operator',...
    'Units','normalized','Position',[.03 .95 .4 .026*uct],...
    'HorizontalAlignment','left');
dom = uicontrol('Parent',icp,'BackgroundColor',bgc1,...
    'String','Please select a 2D domain',...
    'Units','normalized','Position',[.0345 .91 .31 .03*uct],...
    'Callback',{@dom_Callback});

grp = uipanel('Parent',icp,'Title','Discretization settings',...
    'BackgroundColor',bgc,...
    'Units','normalized','Position',[.03 .56 .32 .32]);

nd1 = uicontrol('Parent',grp,'BackgroundColor',bgc,'Style','text',...
    'String','Number of nodes',...
    'Units','normalized','Position',[.03 .85 .65 .09*uct],...
    'HorizontalAlignment','left');
ur1 = uicontrol('Parent',grp,'BackgroundColor',bgc,'Style','text',...
    'String','Uniform refinements',...
    'Units','normalized','Position',[.03 .68 .65 .09*uct],...
    'HorizontalAlignment','left');
rc1 = uicontrol('Parent',grp,'BackgroundColor',bgc,'Style','text',...
    'String','Refinement coefficient',...
    'Units','normalized','Position',[.03 .51 .65 .09*uct],...
    'HorizontalAlignment','left');
et1 = uicontrol('Parent',grp,'BackgroundColor',bgc,'Style','text',...
    'String','Estimator type',...
    'Units','normalized','Position',[.03 .34 .65 .09*uct],...
    'HorizontalAlignment','left');
ei1 = uicontrol('Parent',grp,'BackgroundColor',bgc,'Style','text',...
    'String','Estimator index array',...
    'Units','normalized','Position',[.03 .17 .65 .09*uct],...
    'HorizontalAlignment','left');

nm=10000;
c=0;
cr=0.01;
vps=1;
p1=[1 2 3];

ndi = uicontrol('Parent',grp,'BackgroundColor',bgc1,'Style','edit',...
    'String',nm,...
    'Units','normalized','Position',[.71 .835 .26 .12*uct],...
    'Callback',{@ndi_Callback});
uri = uicontrol('Parent',grp,'BackgroundColor',bgc1,'Style','edit',...
    'String',c,...
    'Units','normalized','Position',[.71 .665 .26 .12*uct],...
    'Callback',{@uri_Callback});
rci = uicontrol('Parent',grp,'BackgroundColor',bgc1,'Style','edit',...
    'String',cr,...
    'Units','normalized','Position',[.71 .495 .26 .12*uct],...
    'Callback',{@rci_Callback});
s=[.71 .32 .26 .12*uct]; if ismac, s=[.69 .32 .3 .12*uct]; end
eti = uicontrol('Parent',grp,'BackgroundColor',bgc1,'Style','popupmenu',...
    'String',{'     Q1 ','     Q2 ','     Q3 ', '     L1 ','     L2 '},...
    'Units','normalized','Position',s,...
    'Callback',{@eti_Callback});
eii = uicontrol('Parent',grp,'BackgroundColor',bgc1,'Style','edit',...
    'String',num2str(p1),...
    'Units','normalized','Position',[.03 .03 .94 .12*uct],...
    'Callback',{@eii_Callback});

mgp = uipanel('Parent',icp,'Title','Multigrid linear solver',...
    'BackgroundColor',bgc,...
    'Units','normalized','Position',[.03 .38 .32 .17]);

vc1 = uicontrol('Parent',mgp,'BackgroundColor',bgc,'Style','text',...
    'String','Max. number of iterations',...
    'Units','normalized','Position',[.03 .72 .65 .19*uct],...
    'HorizontalAlignment','left');
ps1 = uicontrol('Parent',mgp,'BackgroundColor',bgc,'Style','text',...
    'String','Presmoothing and postsmoothing steps',...
    'Units','normalized','Position',[.03 .3 .65 .35*uct],...
    'HorizontalAlignment','left');
cg1 = uicontrol('Parent',mgp,'BackgroundColor',bgc,'Style','checkbox',...
    'String','  cg-preconditioning  ',...
    'Units','normalized','Position',[.03 .06 .94 .19*uct],...
    'HorizontalAlignment','left',...
    'Callback',{@cg1_Callback});

vk1=6;
vkk=2;

vci = uicontrol('Parent',mgp,'BackgroundColor',bgc1,'Style','edit',...
    'String',vk1,...
    'Units','normalized','Position',[.71 .67 .26 .25*uct],...
    'Callback',{@vci_Callback});
psi = uicontrol('Parent',mgp,'BackgroundColor',bgc1,'Style','edit',...
    'String',vkk,...
    'Units','normalized','Position',[.71 .35 .26 .25*uct],...
    'Callback',{@psi_Callback});

esp = uipanel('Parent',icp,'Title','Eigensolver parameters',...
    'BackgroundColor',bgc,...
    'Units','normalized','Position',[.03 .09 .32 .28]);

pv1 = uicontrol('Parent',esp,'BackgroundColor',bgc,'Style','text',...
    'String','PINVIT(k,s) index',...
    'Units','normalized','Position',[.03 .83 .65 .1*uct],...
    'HorizontalAlignment','left');
pv2 = uicontrol('Parent',esp,'BackgroundColor',bgc,'Style','text',...
    'String','(Block) Preconditioned Steepest Descent: (B)PSD',...
    'Units','normalized','Position',[.03 .56 .94 .2*uct],...
    'HorizontalAlignment','left');
ts1 = uicontrol('Parent',esp,'BackgroundColor',bgc,'Style','text',...
    'String','Tightening factor 1',...
    'Units','normalized','Position',[.03 .44 .65 .1*uct],...
    'HorizontalAlignment','left');
ts2 = uicontrol('Parent',esp,'BackgroundColor',bgc,'Style','text',...
    'String','Tightening factor  2',...
    'Units','normalized','Position',[.03 .25 .65 .1*uct],...
    'HorizontalAlignment','left');
pv3 = uicontrol('Parent',esp,'BackgroundColor',bgc,'Style','text',...
    'String','Variants for k>2',...
    'Units','normalized','Position',[.03 .06 .65 .1*uct],...
    'HorizontalAlignment','left');

psdk=2;
psds=9;
rho=0.05;
rho1=0.95;
psdv=1;

pvi = uicontrol('Parent',esp,'BackgroundColor',bgc1,'Style','edit',...
    'String',psdk,...
    'Units','normalized','Position',[.71 .81 .12 .14*uct],...
    'Callback',{@pvi_Callback});
pvj = uicontrol('Parent',esp,'BackgroundColor',bgc1,'Style','edit',...
    'String',psds,...
    'Units','normalized','Position',[.85 .81 .12 .14*uct],...
    'Callback',{@pvj_Callback});
tsi = uicontrol('Parent',esp,'BackgroundColor',bgc1,'Style','edit',...
    'String',rho,...
    'Units','normalized','Position',[.71 .42 .26 .14*uct],...
    'Callback',{@tsi_Callback});
tsj = uicontrol('Parent',esp,'BackgroundColor',bgc1,'Style','edit',...
    'String',rho1,...
    'Units','normalized','Position',[.71 .23 .26 .14*uct],...
    'Callback',{@tsj_Callback});
pvk = uicontrol('Parent',esp,'BackgroundColor',bgc1,'Style','edit',...
    'String',psdv,...
    'Units','normalized','Position',[.71 .04 .26 .14*uct],...
    'Callback',{@pvk_Callback});

soi = uicontrol('Parent',icp,'BackgroundColor',bgc1,'Style','popupmenu',...
    'String',{'   draw all eigenfunctions ','   draw only 1 eigenfunction ',...
              '   do not draw eigenfunctions ', '   reduced output '},...
    'Units','normalized','Position',[.0345 .03 .31 .03*uct],...
    'Callback',{@soi_Callback});

initfig = axes('Parent',icp,...
    'Units','normalized','Position',[.39 .24 .59 .7]);
s=1; if ismac, s=uct/1.2; end
com = uicontrol('Parent',icp,'BackgroundColor',bgc1,...
    'String','Start Computation',...
    'Units','normalized','Position',[.815 .03 .166 .12*s],...
    'Callback',{@com_Callback});

cominfo = uicontrol('Parent',icp,'BackgroundColor',bgc,'Style','text',...
    'Units','normalized','Position',[.375 .03 .43 .11*s]);

rep = uipanel('Parent',bgp,'Title','Results',...
    'BackgroundColor',bgc,...
    'Units','normalized','Position',[.58 .02 .41 .96]);

ccr = uicontrol('Parent',rep,'BackgroundColor',bgc1,...
    'String','Computational costs',...
    'Units','normalized','Position',[.03 .104 .29 .036*uct],...
    'Callback',{@ccr_Callback});

trr = uicontrol('Parent',rep,'BackgroundColor',bgc1,...
    'String','Triangulation',...
    'Units','normalized','Position',[.35 .104 .29 .036*uct],...
    'Callback',{@trr_Callback});

spr = uicontrol('Parent',rep,'BackgroundColor',bgc1,...
    'String','Sparsity pattern',...
    'Units','normalized','Position',[.67 .104 .29 .036*uct],...
    'Callback',{@spr_Callback});

rst=0; if ispc, rst=.005; end

evr = uicontrol('Parent',rep,'BackgroundColor',bgc1,...
    'String','Conv.  Ritz values',...
    'Units','normalized','Position',[.03 .04 .23 .036*uct],...
    'Callback',{@evr_Callback});

evi=1;

evs = uicontrol('Parent',rep,'BackgroundColor',bgc1,'Style','edit',...
    'String',evi,...
    'Units','normalized','Position',[.255+rst .04 .065-rst .0375*uct],...
    'Callback',{@evs_Callback});

efr = uicontrol('Parent',rep,'BackgroundColor',bgc1,...
    'String','Eigenfunctions',...
    'Units','normalized','Position',[.35 .04 .23 .036*uct],...
    'Callback',{@efr_Callback});

efi=1;

efs = uicontrol('Parent',rep,'BackgroundColor',bgc1,'Style','edit',...
    'String',efi,...
    'Units','normalized','Position',[.575+rst .04 .065-rst .0375*uct],...
    'Callback',{@efs_Callback});

rer = uicontrol('Parent',rep,'BackgroundColor',bgc1,...
    'String','Residuals',...
    'Units','normalized','Position',[.67 .04 .23 .036*uct],...
    'Callback',{@rer_Callback});

rei=1;

res = uicontrol('Parent',rep,'BackgroundColor',bgc1,'Style','edit',...
    'String',rei,...
    'Units','normalized','Position',[.895+rst .04 .065-rst .0375*uct],...
    'Callback',{@res_Callback});

resfig = uipanel('Parent',rep,'BorderType','none',...
    'BackgroundColor',bgc,...
    'Units','normalized','Position',[.025 .17 .95 .82]);

set(ccr, 'enable', 'off')
set(trr, 'enable', 'off')
set(spr, 'enable', 'off')
set(evr, 'enable', 'off')
set(efr, 'enable', 'off')
set(rer, 'enable', 'off')
set(evs, 'enable', 'inactive')
set(efs, 'enable', 'inactive')
set(res, 'enable', 'inactive')

s=get(fig,'Position');
s=min(s(3)/sc(3),s(4)/sc(4))*fst;
fs=13.5*s; fs1=15*s; fs2=25*s;
set(icp,'FontSize',fs1)
set(rep,'FontSize',fs1)
set(get(icp,'Children'),'FontSize',fs)
set(get(grp,'Children'),'FontSize',fs)
set(get(mgp,'Children'),'FontSize',fs)
set(get(esp,'Children'),'FontSize',fs)
set(get(rep,'Children'),'FontSize',fs)

axes(initfig)
logo=imread('./utilities/logo.png'); imshow(logo); s=size(logo);
text(s(1)*0.03,36,'Welcome  to  the  {\color{blue}AMP E}igensolver','FontSize',fs2,'FontWeight','bold')
text(s(1)*0.35,s(2)*0.9,'{\color{blue}A}daptive','FontSize',fs2,'FontWeight','bold')
text(s(1)*0.35,s(2)*0.95,'{\color{blue}M}ultigrid','FontSize',fs2,'FontWeight','bold')
text(s(1)*0.35,s(2),'{\color{blue}P}reconditioned','FontSize',fs2,'FontWeight','bold')
text(s(1)*0.35,s(2)*1.05,'{\color{blue}E}igensolver','FontSize',fs2,'FontWeight','bold')


    function bgp_ResizeFcn(source,eventdata)
        s=get(fig,'Position');
        s=min(s(3)/sc(3),s(4)/sc(4))*fst;
        fs=13.5*s; fs1=15*s; fs2=25*s;
        set(icp,'FontSize',fs1)
        set(rep,'FontSize',fs1)
        set(get(icp,'Children'),'FontSize',fs)
        set(get(grp,'Children'),'FontSize',fs)
        set(get(mgp,'Children'),'FontSize',fs)
        set(get(esp,'Children'),'FontSize',fs)
        set(get(rep,'Children'),'FontSize',fs)
        if domainselected==0
            axes(initfig)
            logo=imread('./utilities/logo.png'); imshow(logo); s=size(logo);
            text(s(1)*0.15,36,'Welcome  to  the  {\color{blue}AMP E}igensolver','FontSize',fs2,'FontWeight','bold')
            text(s(1)*0.35,s(2)*0.9,'{\color{blue}A}daptive','FontSize',fs2,'FontWeight','bold')
            text(s(1)*0.35,s(2)*0.95,'{\color{blue}M}ultigrid','FontSize',fs2,'FontWeight','bold')
            text(s(1)*0.35,s(2),'{\color{blue}P}reconditioned','FontSize',fs2,'FontWeight','bold')
            text(s(1)*0.35,s(2)*1.05,'{\color{blue}E}igensolver','FontSize',fs2,'FontWeight','bold')
        else
            initialgrid
        end
        switch efdrawn
            case 1
                if get(soi,'Value')==1
                    fsv=min(fs1,fs*spf);
                    for j=1:length(spv)
                        set(spv(j),'FontSize',fsv)
                    end
                else
                    set(spv,'FontSize',fs1)
                end
            case 2
                ccr_Callback
            case 3
                set(sp,'FontSize',fs)
            case 4
                set(sp,'FontSize',fs)
                set(spv,'FontSize',fs)
            case 5
                evr_Callback
            case 6
                set(sp,'FontSize',fs)
            case 7
                rer_Callback
        end
    end

    function dom_Callback(source,eventdata)
        [domain,domains]=uigetfile('./domains/*.m','Select a domain');
        if domain==0
            axes(initfig)
            imshow('./utilities/logo.png');
            domainselected=0;
            set(cominfo,'String',{' ';'no domain selected'},'ForegroundColor','b');
            return
        end
        run([domains,domain]);
        initial=load('./domains/domain.mat');
        inx=initial.x; iny=initial.y; invt=initial.v;
        ine=initial.e; inep=initial.ep; intr=initial.tr;
        intrp=initial.trp; cc=initial.cc; ci=initial.ci;
        domainselected=1;
        inv1=ones(1,length(inx)); inv1(invt)=2;
        inh=0.1*sqrt((inx(1)-inx(2))^2+(iny(1)-iny(2))^2);
        inaxis=[min(inx)-inh max(inx)+inh min(iny)-inh max(iny)+inh];
        initialgrid
        set(cominfo, 'String', {' '; 'new domain selected'}, 'ForegroundColor', 'b');
    end

    function initialgrid
        axes(initfig)
        cla
        ttt=plot(inx,iny,'k.');
        set(ttt,'ButtonDownFcn',{@initialgrid1});
        hold on
        for i=1:size(ine,1)
            xc=sum(inx(ine(i,:)))/2; yc=sum(iny(ine(i,:)))/2;
            br=0;
            for j=1:i-1
                if (xc-sum(inx(ine(j,:)))/2)^2+(yc-sum(iny(ine(j,:)))/2)^2<1e-10
                    br=1; break;
                end
            end
            epi1=inep(i,1);
            if epi1<1
                cl='k';
            else
                cl=cl1;
            end
            if br==0
                ttt=plot(inx(ine(i,:)),iny(ine(i,:)),'Color',cl);
                set(ttt,'ButtonDownFcn',{@initialgrid1});
                ttt=text(xc,yc,num2str(i),'FontSize',fs1,'FontWeight','bold');
                set(ttt,'ButtonDownFcn',{@initialgrid1});
            else
                ttt=text(xc+inh,yc,['(',num2str(i),')'],'Color',cl,'FontSize',fs1,'FontWeight','bold');
                set(ttt,'ButtonDownFcn',{@initialgrid1});
            end
        end
        for i=1:length(inx)
            if inv1(i)==1
                ttt=plot(inx(i),iny(i),'o','Color',cl1,'MarkerSize',9,'LineWidth',2);
                set(ttt,'ButtonDownFcn',{@initialgrid1});
            end
        end
        hold off
        axis equal
        axis(inaxis)
        set(gca,'XColor',bgc,'YColor',bgc)
        ttt=title('Initial grid','Color',cl1,'FontSize',fs1,'FontWeight','bold');
        set(ttt,'ButtonDownFcn',{@initialgrid1});
        ttt=xlabel({['{\color{black}{\color[rgb]{0 0.3 1}Blue} circles/lines}'...
            '{\color{black} are drawn for {\color[rgb]{0 0.3 1}fixed} nodes/edges.}']; ' '; ...
            ['{\color{black}Two edge numbers are printed for overlapping edges.}']}, ...
            'HorizontalAlignment','center','FontSize',fs);
        if ismac
            spw=get(ttt,'Position'); spw(1)=spw(1)+0.15; set(ttt,'Position',spw);
        end
        set(ttt,'ButtonDownFcn',{@initialgrid1});
        set(gca,'ButtonDownFcn',{@initialgrid1});
    end

    function initialgrid1(source,eventdata)
        if strcmp(get(fig,'SelectionType'),'alt')
            figure('NumberTitle','off','Name','Initial grid'); axes
            plot(inx,iny,'k.');
            hold on
            for i=1:size(ine,1)
                xc=sum(inx(ine(i,:)))/2; yc=sum(iny(ine(i,:)))/2;
                br=0;
                for j=1:i-1
                    if (xc-sum(inx(ine(j,:)))/2)^2+(yc-sum(iny(ine(j,:)))/2)^2<1e-10
                        br=1; break;
                    end
                end
                epi1=inep(i,1);
                if epi1<1
                    cl='k';
                else
                    cl=cl1;
                end
                if br==0
                    plot(inx(ine(i,:)),iny(ine(i,:)),'Color',cl);
                    text(xc,yc,num2str(i),'FontSize',fs1,'FontWeight','bold');
                else
                    text(xc+inh,yc,['(',num2str(i),')'],'Color',cl,'FontSize',fs1,'FontWeight','bold');
                end
            end
            for i=1:length(inx)
                if inv1(i)==1
                    plot(inx(i),iny(i),'o','Color',cl1,'MarkerSize',9,'LineWidth',2);
                end
            end
            hold off
            axis equal
            axis(inaxis)
            axis off
        end
    end

    function initupt
        set(cominfo, 'String', {' '; 'initial setting updated'}, 'ForegroundColor', 'b');
    end

    function ndi_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        if isnan(nms)
            errordlg('Input must be a number','Error');
        elseif nms<=0
            errordlg('Input must be positive','Error');
        elseif (nms-round(nms))~=0
            errordlg('Input must be an integer','Error');
        else
            nm=nms; initupt
            if nm<30000
                set(soi,'Value',1);
            elseif nm<1e6
                set(soi,'Value',2);
            elseif nm<1e7
                set(soi,'Value',3);
            else
                set(soi,'Value',4);
                warndlg({'Input might be too large for some PCs'},'Warning');
            end
        end
        set(source,'String',nm);
    end

    function uri_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        if isnan(nms)
            errordlg('Input must be a number','Error');
        elseif nms<0
            errordlg('Input must be non-negative','Error');
        elseif (nms-round(nms))~=0
            errordlg('Input must be an integer','Error');
        else
            c=nms; initupt
        end
        set(source,'String',c);
    end

    function rci_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        if isnan(nms)
            errordlg('Input must be a number','Error');
        elseif nms<=0 || nms>=1
            errordlg('Input must be between 0 and 1','Error');
        else
            cr=nms; initupt
        end
        set(source,'String',cr);
    end

    function eti_Callback(source,eventdata)
        vps=get(source,'Value'); initupt
    end

    function eii_Callback(source,eventdata)
        str1=get(source,'String');
        if length(str1)==1
            nms=str2num(str1(1));
        elseif length(str1)>1
            p1k=0; nms=[];
            for i=1:length(str1)-1
                str1i=str1(i);
                if str1i~=' '
                    p1k=p1k*10+str2num(str1i);
                else
                    if p1k~=0, nms=[nms p1k]; end
                    p1k=0;
                end
            end
            i=i+1;
            str1i=str1(i);
            if str1i~=' '
                p1k=p1k*10+str2num(str1i);
            end
            if p1k~=0, nms=[nms p1k]; end
        end
        nms=sort(nms); nms=unique(nms);
        if isempty(nms)
            errordlg('Input must be positive integers','Error');
        else
            p1=nms; initupt
        end
        set(source,'String',num2str(p1));
        psds=max([p1 psds]);
        set(pvj,'String',psds);
    end

    function vci_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        if isnan(nms)
            errordlg('Input must be a number','Error');
        elseif nms<=0
            errordlg('Input must be positive','Error');
        elseif (nms-round(nms))~=0
            errordlg('Input must be an integer','Error');
        else
            vk1=nms; initupt
        end
        set(source,'String',vk1);
    end

    function psi_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        if isnan(nms)
            errordlg('Input must be a number','Error');
        elseif nms<=0
            errordlg('Input must be positive','Error');
        elseif (nms-round(nms))~=0
            errordlg('Input must be an integer','Error');
        else
            vkk=nms; initupt
        end
        set(source,'String',vkk);
    end

    function cg1_Callback(source,eventdata)
        initupt
    end

    function pvi_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        if isnan(nms)
            errordlg('Input must be a number','Error');
        elseif nms<=0
            errordlg('Input must be positive','Error');
        elseif (nms-round(nms))~=0
            errordlg('Input must be an integer','Error');
        else
            psdk=nms; initupt
        end
        set(source,'String',psdk);
        if psdk==1
            set(pv2, 'String', '(Block) Preconditioned Inverse Iteration: (B)PINVIT');
        elseif psdk==2
            set(pv2, 'String', '(Block) Preconditioned Steepest Descent: (B)PSD');
        elseif psdk==3
            set(pv2, 'String', 'Locally Optimized (Block) Preconditioned CG: LO(B)PCG');
        else
            set(pv2, 'String', 'Generalization of LO(B)PCG using more previous subspaces');
        end
    end

    function pvj_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        nms1=max(p1);
        if isnan(nms)
            errordlg('Input must be a number','Error');
        elseif nms<=0
            errordlg('Input must be positive','Error');
        elseif nms<nms1
            psds=nms1;
            errordlg({['Input must be at least ', num2str(nms1)]; ...
                'according to estimator index array'},'Error');
        elseif (nms-round(nms))~=0
            errordlg('Input must be an integer','Error');
        else
            psds=nms; initupt
        end
        set(source,'String',psds);
    end

    function tsi_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        if isnan(nms)
            errordlg('Input must be a number','Error');
        elseif nms<=0 || nms>=1
            errordlg('Input must be between 0 and 1','Error');
        else
            rho=nms; initupt
        end
        set(source,'String',rho);
    end

    function tsj_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        if isnan(nms)
            errordlg('Input must be a number','Error');
        elseif nms<=0 || nms>=1
            errordlg('Input must be between 0 and 1','Error');
        else
            rho1=nms; initupt
        end
        set(source,'String',rho1);
    end

    function pvk_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        if isnan(nms)
            errordlg('Input must be a number','Error');
        elseif nms<1 || nms>3
            errordlg('Input must be between 1 and 3','Error');
        elseif (nms-round(nms))~=0
            errordlg('Input must be an integer','Error');
        else
            psdv=nms; initupt
        end
        set(source,'String',psdv);
    end

    function soi_Callback(source,eventdata)
        initupt
    end

    function com_Callback(source,eventdata)
        if domainselected==0
            errordlg('no domain selected','Error');
            return;
        end
        set(resfig,'ButtonDownFcn',{});
        sp=subplot(1,1,1,'Parent',resfig);
        plot(sp,1,1); axis off; cla; 
        set(sp,'ButtonDownFcn',{});
        efdrawn=0;
        set(ccr, 'enable', 'off')
        set(trr, 'enable', 'off')
        set(spr, 'enable', 'off')
        set(evr, 'enable', 'off')
        set(efr, 'enable', 'off')
        set(rer, 'enable', 'off')
        evi=1; efi=1; rei=p1(1);
        set(evs, 'enable', 'inactive', 'String', evi)
        set(efs, 'enable', 'inactive', 'String', efi)
        set(res, 'enable', 'inactive', 'String', rei)
        x=inx; y=iny; v=invt; 
        e=ine; ep=inep; tr=intr; trp=intrp;
        nmax=max(nm*3,1500000);
        lvmax=150;
        if get(soi,'Value')~=4, on=2; else on=1; end
        cg=get(cg1,'Value');
        n=length(x);
        em=size(e,1);
        m=size(tr,1);
        vn=length(v);
        cn=length(ci);
        p1n=length(p1);
        f=fopen('./fortran_io/domain','w');
        fprintf(f,'%10d\n',n);
        fprintf(f,'%10d\n',em);
        fprintf(f,'%10d\n',m);
        fprintf(f,'%10d\n',vn);
        fprintf(f,'%10d\n',cn);
        for i=1:n
            fprintf(f,'%25.15e\n',x(i));
        end
        for i=1:n
            fprintf(f,'%25.15e\n',y(i));
        end
        for i=1:vn
            fprintf(f,'%10d\n',v(i));
        end
        for i=1:cn
            fprintf(f,'%10d\n',ci(i));
        end
        for i=1:cn
            fprintf(f,'%25.15e%25.15e%25.15e\n',cc(i,1:3));
        end
        for i=1:em
            fprintf(f,'%10d%10d\n',e(i,1:2));
        end
        for i=1:em
            fprintf(f,'%10d%10d\n',ep(i,1:2));
        end
        for i=1:m
            fprintf(f,'%10d%10d%10d\n',tr(i,1:3));
        end
        for i=1:m
            fprintf(f,'%10d%10d\n',trp(i,1:2));
        end
        fclose(f);
        f=fopen('./fortran_io/input','w');
        fprintf(f,'%10d\n',nm);
        fprintf(f,'%10d\n',nmax);
        fprintf(f,'%10d\n',lvmax);
        fprintf(f,'%10d\n',psds);
        fprintf(f,'%10d\n',p1n);
        for i=1:p1n
            fprintf(f,'%10d\n',p1(i));
        end
        fprintf(f,'%10.5f\n',cr);
        fprintf(f,'%10d\n',vps);
        fprintf(f,'%10d\n',cg);
        fprintf(f,'%10d\n',on);
        fprintf(f,'%10d\n',c);
        fprintf(f,'%10.5f\n',rho);
        fprintf(f,'%10.5f\n',rho1);
        fprintf(f,'%10d\n',vk1);
        fprintf(f,'%10d\n',vkk);
        fprintf(f,'%10d\n',psdk);
        switch psdv
            case 1
                fprintf(f,'%10d\n',1);
            case 2
                fprintf(f,'%10d\n',0);
            case 3
                fprintf(f,'%10d\n',2);
        end
        fclose(f);
        set(cominfo, 'String', {' ';...
            'Computation started. See command window for details.'},...
            'ForegroundColor', 'b');
        set(fig,'Pointer','watch'); drawnow
        tic
        run('./fortran_io/fortrancom')
        set(fig,'Pointer','arrow')
        fprintf(' program completed\n')
        f=fopen('./fortran_io/output','r');
        spw=fread(f,1,'int');
        n=fread(f,1,'int');
        em=fread(f,1,'int');
        m=fread(f,1,'int');
        lv=fread(f,1,'int');
        v1n=fread(f,1,'int');
        theta=fread(f,[2 psds],'double'); theta=theta(2,:);
        nn=fread(f,[5 lv],'int'); nn=nn(3:5,:);
        tt=fread(f,[3 lv],'single'); tt=tt(3,:);
        tt1=zeros(1,lv);
        tt1(1)=tt(1);
        for i=2:lv
            tt1(i)=tt1(i-1)+tt(i);
        end
        thetan=nn(3,lv);
        thetav=zeros(thetan,psds);
        for j=1:psds
            thetaj=fread(f,[2 thetan],'double');
            thetav(:,j)=thetaj(2,:)';
        end
        for i=1:p1n
            rn1=fread(f,[4 lv],'double');
            rn.nr(i).rn=rn1(2:4,:);
        end
        set(cominfo, 'String',...
            {['Computation finished with ',num2str(psds),' eigenvalues, '],...
            [num2str(n),' nodes, ',num2str(em),' edges and ',num2str(m),' triangles. '],...
            ['Total time : ',num2str(toc,'%.2f'),' sec.'],...
            ['CPU time for eigensolver : ',num2str(tt1(length(tt1)),'%.2f'), ' sec.']},...
            'ForegroundColor','b','FontSize',fs);
        if on==2
            wb1=3; wb2=35; wb3=30; wb4=31;
            sp=subplot(1,1,1,'Parent',resfig);
            plot(sp,[wb1 wb2 wb2 wb1 wb1],[wb3 wb3 wb4 wb4 wb3]);
            title('loading triangles','Color','b','FontSize',fs1)
            axis equal; axis([wb1 wb2 wb1 wb2]); axis off; drawnow
            hold on
            set(fig,'Pointer','watch'); drawnow
            tr=fread(f,[5 m],'int');
            tr=tr(3:5,:)'; wb5=wb1*0.75+wb2*0.25;
            fill([wb1 wb5 wb5 wb1 wb1],[wb3 wb3 wb4 wb4 wb3],'b','EdgeColor','b','Parent',sp);
            title('loading edges','Color','b','FontSize',fs1)
            axis equal; axis([wb1 wb2 wb1 wb2]); axis off; drawnow
            e=fread(f,[4 em],'int');
            e=e(3:4,:)'; wb5=wb1*0.5+wb2*0.5;
            fill([wb1 wb5 wb5 wb1 wb1],[wb3 wb3 wb4 wb4 wb3],'b','EdgeColor','b','Parent',sp);
            title('loading nodes','Color','b','FontSize',fs1)
            axis equal; axis([wb1 wb2 wb1 wb2]); axis off; drawnow
            xy=fread(f,[3 n],'double');
            x=xy(2,:)'; y=xy(3,:)'; wb5=wb1*0.25+wb2*0.75;
            fill([wb1 wb5 wb5 wb1 wb1],[wb3 wb3 wb4 wb4 wb3],'b','EdgeColor','b','Parent',sp);
            title('loading eigenfunctions','Color','b','FontSize',fs1)
            axis equal; axis([wb1 wb2 wb1 wb2]); axis off; drawnow
            u=zeros(n,psds);
            for j=1:psds
                uj=fread(f,[2 n],'double');
                u(:,j)=uj(2,:)';
            end
            v1=fread(f,[3 v1n],'int'); v1=v1(3,:);
            for k=1:m
                s1=tr(k,1); s2=tr(k,2); s3=tr(k,3);
                t11=e(s1,1); t12=e(s1,2);
                t21=e(s2,1); t22=e(s2,2);
                t31=e(s3,1); t32=e(s3,2);
                if t11==t21
                    t2=t11; t1=t12; t3=t22;
                else
                    if t12==t21
                        t2=t12; t1=t11; t3=t22;
                    else
                        if t11==t31 || t11==t32
                            t1=t11; t2=t12;
                        else
                            t1=t12; t2=t11;
                        end
                        t3=t21;
                    end
                end
                tr(k,1)=t1; tr(k,2)=t2; tr(k,3)=t3;
            end
            fill([wb1 wb2 wb2 wb1 wb1],[wb3 wb3 wb4 wb4 wb3],'b','EdgeColor','b','Parent',sp);
            if get(soi,'Value')==3
                title('data loaded','Color','b','FontSize',fs1)
            else
                title('drawing','Color','b','FontSize',fs1)
            end
            axis equal; axis([wb1 wb2 wb1 wb2]); axis off; drawnow
            set(fig,'Pointer','arrow')
            hold off;
        end
        efdrawn=1;
        efdraw
    end

    function efdraw
        set(fig,'Pointer','watch'); drawnow
        s2=ceil(sqrt(psds)); s1=ceil(psds/s2); spf=3/s2;
        if get(soi,'Value')==1
            spv=zeros(1,psds); fsv=min(fs1,fs*spf);
            tempfv=zeros(1,psds); spv1=spv;
            for j=1:psds
                u1=u(:,j); [u11,i]=max(abs(u1));
                u1=u1/u1(i);
                sp=subplot(s1,s2,j,'Parent',resfig); spv1(j)=sp;
                tempfv(j)=trisurf(tr,x(1:n),y(1:n),u1,'Parent',sp);
                shading interp
                axis equal
                axisv=[min(x) max(x) min(y) max(y) -1 1];
                axis(axisv)
                set(gca,'XColor',bgc,'YColor',bgc,'ZColor',bgc)
                view(3)
                camlight
                lighting gouraud
                spv(j)=title(['Eigenfunction # ', num2str(j)],'Color','b','FontSize',fsv);
                drawnow
            end
            for j=1:psds
                set(tempfv(j),'ButtonDownFcn',{@resfig_ButtonDownFcn});
                set(spv1(j),'ButtonDownFcn',{@resfig_ButtonDownFcn});
            end
            set(resfig,'ButtonDownFcn',{@resfig_ButtonDownFcn});
        elseif get(soi,'Value')==2
            u1=u(:,1); [u11,i]=max(abs(u1));
            u1=u1/u1(i);
            sp=subplot(1,1,1,'Parent',resfig);
            tempf=trisurf(tr,x(1:n),y(1:n),u1,'Parent',sp);
            shading interp
            axis equal tight off
            view(3)
            camlight
            lighting gouraud
            spv=title('Eigenfunction # 1','Color','b','FontSize',fs1);
            set(tempf,'ButtonDownFcn',{@resfig_ButtonDownFcn});
            set(gca,'ButtonDownFcn',{@resfig_ButtonDownFcn});
            set(resfig,'ButtonDownFcn',{@resfig_ButtonDownFcn});
        elseif get(soi,'Value')==3
            spv=title('data loaded','Color','b','FontSize',fs1);
        else
            spv=title('reduced data loaded','Color','b','FontSize',fs1);
        end
        drawnow; set(fig,'Pointer','arrow')
        set(ccr, 'enable', 'on')
        set(evr, 'enable', 'on')
        set(rer, 'enable', 'on')
        set(evs, 'enable', 'on')
        set(res, 'enable', 'on')
        if on==2
            set(trr, 'enable', 'on')
            set(spr, 'enable', 'on')
            set(efr, 'enable', 'on')
            set(efs, 'enable', 'on')
        end
    end

    function evs_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        s=ceil(psds/3);
        if isnan(nms)
            evi=1;
            errordlg('Input must be a number','Error');
        elseif nms<1 || nms>s
            evi=1;
            errordlg({['Input must be between 1 and ' num2str(s)];
               'according to subspace dimension'},'Error');
        elseif (nms-round(nms))~=0
            evi=1;
            errordlg('Input must be an integer','Error');
        else
            evi=nms; evr_Callback(source,eventdata)
        end
        set(source,'String',evi);
    end

    function efs_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        if isnan(nms)
            efi=1;
            errordlg('Input must be a number','Error');
        elseif nms<1 || nms>psds
            efi=1;
            errordlg({['Input must be between 1 and ' num2str(psds)]; ...
               'according to subspace dimension'},'Error');
        elseif (nms-round(nms))~=0
            efi=1;
            errordlg('Input must be an integer','Error');
        else
            efi=nms; efr_Callback(source,eventdata)
        end
        set(source,'String',efi);
    end

    function res_Callback(source,eventdata)
        nms=str2double(get(source,'String'));
        p1n=length(p1);
        p1i=0; for i=1:p1n, if p1(i)==nms, p1i=1; end, end
        if isnan(nms)
            rei=p1(1);
            errordlg('Input must be a number','Error');
        elseif p1i==0
            rei=p1(1);
            errordlg('Input must belong to the estimator index array','Error');
        elseif (nms-round(nms))~=0
            rei=p1(1);
            errordlg('Input must be an integer','Error');
        else
            rei=nms; rer_Callback(source,eventdata)
        end
        set(source,'String',rei);
    end

    function ccr_Callback(source,eventdata)
        set(fig,'Pointer','watch'); drawnow
        efdrawn=2;
        sp=subplot(1,1,1,'Parent',resfig);
        tempf=loglog(sp,nn(1,:),tt,'-*',nn(1,:),tt1,'-*');
        set(sp,'FontSize',fs)
        axis square
        ll=legend('time per level','total time','Location','northwest');
        lp=get(ll,'Position'); set(ll,'Position',[lp(1)*1.01 lp(2)*0.99 lp(3) lp(4)],'FontSize',fs);
        xlabel('number of nodes')
        set(sp,'ButtonDownFcn', {@resfig_ButtonDownFcn}); set(tempf,'ButtonDownFcn',{@resfig_ButtonDownFcn});
        drawnow; set(fig,'Pointer','arrow')
    end

    function trr_Callback(source,eventdata)
        set(fig,'Pointer','watch'); drawnow
        h=0.1*sqrt((x(1)-x(2))^2+(y(1)-y(2))^2);
        efdrawn=3;
        sp=subplot(1,1,1,'Parent',resfig);
        tempf=trisurf(tr,x(1:n),y(1:n),ones(1,n),'Parent',sp);
        set(sp,'FontSize',fs)
        axis equal
        axisv=[min(x)-h max(x)+h min(y)-h max(y)+h -1 1];
        axis(axisv)
        view(2)
        set(sp,'ButtonDownFcn', {@resfig_ButtonDownFcn}); set(tempf,'ButtonDownFcn',{@resfig_ButtonDownFcn});
        drawnow; set(fig,'Pointer','arrow')
    end

    function spr_Callback(source,eventdata)
        set(fig,'Pointer','watch'); drawnow
        m=length(e); e1=(1:n)';
        A1=sparse([e1; e(:,1); e(:,2)],[e1; e(:,2); e(:,1)],ones(n+2*m,1));
        v2=1:v1(1)-1;
        for i=1:length(v1)-1
            v2=[v2 v1(i)+1:v1(i+1)-1];
        end
        v2=[v2 v1(i+1)+1:n]; n1=length(v2)+1;
        efdrawn=4;
        sp=subplot(1,1,1,'Parent',resfig);
        [e1,e2,a]=find(A1(v2,v2)); n2=length(a);
        tempf=plot(sp,e1,e2,'.','MarkerSize',0.1);
        set(sp,'FontSize',fs)
        axis equal
        axis([0 n1 0 n1])
        axis ij
        spv=xlabel([num2str(n2),' non-zeros']);
        set(spv,'FontSize',fs)
        set(sp,'ButtonDownFcn', {@resfig_ButtonDownFcn}); set(tempf,'ButtonDownFcn',{@resfig_ButtonDownFcn});
        drawnow; set(fig,'Pointer','arrow')
    end

    function evr_Callback(source,eventdata)
        set(fig,'Pointer','watch'); drawnow
        s=psds;
        lambda=thetav(nn(3,end),:);
        thetap=1:nn(3,max(1,end-1));
        thetav1=thetav(thetap,:);
        for i=thetap
            thetav1(i,:)=thetav(i,:)-lambda;
        end
        j=(evi-1)*3+1:min(evi*3,s); evj=j;
        efdrawn=5;
        sp=subplot(1,1,1,'Parent',resfig);
        tempf=semilogy(thetav1(:,j));
        set(sp,'FontSize',fs)
        xlabel('iteration index')
        axis square
        switch length(j)
            case 1
                ll=legend(['\lambda_{',num2str(j(1)),'} : ',num2str(lambda(j(1)),'%.5f')]);
                lp=get(ll,'position'); set(ll,'position',[lp(1)*0.98 lp(2)*0.99 lp(3) lp(4)]);
            case 2
                ll=legend(['\lambda_{',num2str(j(1)),'} : ',num2str(lambda(j(1)),'%.5f')],...
                    ['\lambda_{',num2str(j(2)),'} : ',num2str(lambda(j(2)),'%.5f')]);
                lp=get(ll,'position'); set(ll,'position',[lp(1)*0.98 lp(2)*0.99 lp(3) lp(4)]);
            case 3
                ll=legend(['\lambda_{',num2str(j(1)),'} : ',num2str(lambda(j(1)),'%.5f')],...
                    ['\lambda_{',num2str(j(2)),'} : ',num2str(lambda(j(2)),'%.5f')],...
                    ['\lambda_{',num2str(j(3)),'} : ',num2str(lambda(j(3)),'%.5f')]);
                lp=get(ll,'position'); set(ll,'position',[lp(1)*0.98 lp(2)*0.99 lp(3) lp(4)]);
        end
        set(sp,'ButtonDownFcn', {@resfig_ButtonDownFcn}); set(tempf,'ButtonDownFcn',{@resfig_ButtonDownFcn});
        drawnow; set(fig,'Pointer','arrow')
    end

    function efr_Callback(source,eventdata)
        set(fig,'Pointer','watch'); drawnow
        u1=u(:,efi); [u11,i]=max(abs(u1));
        u1=u1/u1(i);
        efdrawn=6;
        sp=subplot(1,1,1,'Parent',resfig);
        tempf=trisurf(tr,x(1:n),y(1:n),u1,'Parent',sp);
        set(sp,'FontSize',fs)
        shading interp
        axis equal tight
        camlight
        set(sp,'ButtonDownFcn', {@resfig_ButtonDownFcn}); set(tempf,'ButtonDownFcn',{@resfig_ButtonDownFcn});
        drawnow; set(fig,'Pointer','arrow')
    end

    function rer_Callback(source,eventdata)
        set(fig,'Pointer','watch'); drawnow
        i=1; while p1(i)~=rei, i=i+1; end
        rn1=rn.nr(i).rn; 
        pp1=1:lv; pp2=2:lv;
        efdrawn=7;
        sp=subplot(1,1,1,'Parent',resfig);
        tempf=semilogy(sp,pp1,sqrt(rn1(1,pp1)),pp2,rn1(2,pp2),pp2,rn1(3,pp2));
        set(sp,'FontSize',fs)
        xlabel('level index')
        axis square
        ll=legend('residual estimate','stopping criterion','residual norm');
        lp=get(ll,'position'); set(ll,'position',[lp(1)*0.98 lp(2)*0.99 lp(3) lp(4)]);
        set(sp,'ButtonDownFcn', {@resfig_ButtonDownFcn}); set(tempf,'ButtonDownFcn',{@resfig_ButtonDownFcn});
        drawnow; set(fig,'Pointer','arrow')
    end

    function resfig_ButtonDownFcn(source,eventdata)
        if strcmp(get(fig,'SelectionType'),'alt')
            figure('NumberTitle','off','Name','AMPE Results'); axes
            switch efdrawn
                case 1
                    if get(soi,'Value')==1
                        for j=1:psds
                            subplot(s1,s2,j);
                            copyobj(tempfv(j),gca)
                            shading interp
                            axis equal
                            axis(axisv)
                            axis off
                            view(3)
                            camlight
                            title(['Eigenfunction # ', num2str(j)],'Color','b');
                        end
                    elseif get(soi,'Value')==2
                        copyobj(tempf,gca)
                        shading interp
                        axis equal tight off
                        view(3)
                        camlight
                        title('Eigenfunction # 1','Color','b');
                    end
                case 2
                    loglog(1,1); hold on; cla
                    copyobj(tempf,gca); hold off
                    axis square
                    ll=legend('time per level','total time','Location','northwest');
                    lp=get(ll,'Position'); set(ll,'Position',[lp(1)*1.01 lp(2)*0.99 lp(3) lp(4)],'FontSize',fs);
                    xlabel('number of nodes')
                case 3
                    copyobj(tempf,gca)
                    axis equal
                    axis(axisv)
                    view(2)
                    grid on
                case 4
                    copyobj(tempf,gca)
                    axis equal
                    axis([0 n1 0 n1])
                    axis ij
                    box on
                    xlabel([num2str(n2),' non-zeros']);
                case 5
                    semilogy(1,1); hold on; cla
                    copyobj(tempf,gca); hold off
                    axis square
                    j=evj;
                    switch length(j)
                        case 1
                            ll=legend(['\lambda_{',num2str(j(1)),'} : ',num2str(lambda(j(1)),'%.5f')]);
                            lp=get(ll,'position'); set(ll,'position',[lp(1)*0.98 lp(2)*0.99 lp(3) lp(4)]);
                        case 2
                            ll=legend(['\lambda_{',num2str(j(1)),'} : ',num2str(lambda(j(1)),'%.5f')],...
                                ['\lambda_{',num2str(j(2)),'} : ',num2str(lambda(j(2)),'%.5f')]);
                            lp=get(ll,'position'); set(ll,'position',[lp(1)*0.98 lp(2)*0.99 lp(3) lp(4)]);
                        case 3
                            ll=legend(['\lambda_{',num2str(j(1)),'} : ',num2str(lambda(j(1)),'%.5f')],...
                                ['\lambda_{',num2str(j(2)),'} : ',num2str(lambda(j(2)),'%.5f')],...
                                ['\lambda_{',num2str(j(3)),'} : ',num2str(lambda(j(3)),'%.5f')]);
                            lp=get(ll,'position'); set(ll,'position',[lp(1)*0.98 lp(2)*0.99 lp(3) lp(4)]);
                    end
                    xlabel('iteration index')
                case 6
                    copyobj(tempf,gca)
                    shading interp
                    axis equal tight
                    view(3)
                    grid on
                    camlight
                case 7
                    semilogy(1,1); hold on; cla
                    copyobj(tempf,gca); hold off
                    axis square
                    ll=legend('residual estimate','stopping criterion','residual norm');
                    lp=get(ll,'position'); set(ll,'position',[lp(1)*0.98 lp(2)*0.99 lp(3) lp(4)]);
                    xlabel('level index')
            end
        end
    end

    function bgp_ButtonDownFcn(source,eventdata)
        if strcmp(get(fig,'SelectionType'),'normal')
            set(fig,'Menubar','none','Toolbar','none'); set(bgp,'ResizeFcn',{@bgp_ResizeFcn});
        elseif strcmp(get(fig,'SelectionType'),'alt')
            set(fig,'Menubar','figure','Toolbar','figure'); set(bgp,'ResizeFcn',{});
        end
    end
end
